#!/usr/bin/env python

import os
import sys
import glob

from debian import deb822
from xml.dom.minidom import Document
from dateutil.parser import parse as date_parse


def main(basedir):
    e = []
    error = False
    num = 0

    doc = Document()
    events = doc.createElement('data')
    doc.appendChild(events)

    filenames = glob.glob(os.path.join(basedir, '*'))

    for filename in sorted(filenames):
        if e:
            print >>sys.stderr
        print >>sys.stderr, "Reading events from %s" % filename,
        input = file(filename).read().decode('utf-8').split('\n')

        e = []
        para_num = 0
        for para in deb822.Deb822.iter_paragraphs(input, use_apt_pkg=False):
            if 'Title' not in para:
                title = "para %s" % para_num
                e.append("Start-Date should be before End-Date for %s" % title)
            else:
                title = para['Title']
            dates = {}
            for header in ('Date', 'Start-Date', 'End-Date'):
                if header not in para:
                    continue
                try:
                    dates[header] = date_parse(para[header])
                except (TypeError, ValueError):
                    e.append("Invalid date header %s for %s" % (header, title))
            if 'Start-Date' in para and 'End-Date' in para:
                if 'Start-Date' in dates and 'End-Date' in dates:
                    if dates['Start-Date'] > dates['End-Date']:
                        e.append("Start-Date is after End-Date for %s" % title)
            elif 'Start-Date' in para or 'End-Date' in para:
                e.append("Missing Start-Date or End-Date for %s" % title)
            elif 'Date' not in para:
                e.append("Missing date or date range for %s" % title)
            events.appendChild(create_event(doc, para))
            sys.stderr.write('.')
            num += 1
            para_num += 1
        print >>sys.stderr
        if e:
            for error in e:
                print >>sys.stderr, error
            error = True

    if error:
        return 1
    print >>sys.stderr, "Writing %s events" % num

    print '<!-- Generated from %s/* - do not edit -->' % basedir
    print events.toprettyxml(indent='  ').encode('utf-8')


def create_event(doc, para):
    entry = doc.createElement('entry')
    entry.setAttribute('title', para['Title'])

    if 'Start-Date' in para:
        entry.setAttribute('isDuration', 'true')
        entry.setAttribute('start', para['Start-Date'])
        entry.setAttribute('end', para['End-Date'])
    else:
        entry.setAttribute('start', para['Date'])

    if 'Source' in para:
        text = doc.createTextNode('<a href="%s" target="debian-timeline-source">Source</a>' % para['Source'])
        entry.appendChild(text)

    return entry

if __name__ == '__main__':
    sys.exit(main(sys.argv[1]))



#!/usr/bin/env python

## Originally written by Rafael Laboissiere in 2015.
##
## This script is hereby placed into the public domain, no rights reserved.

## Use this script for updating file data/events/bugs with more recent
## bugs.  Launch the script from the top level directory, like this:
##
##    ./update-bug-list.py
##
## The file data/events/bugs will be overwritten.

import re
import SOAPpy
from debian import deb822
from datetime import datetime

### This is the granularity for inclusion of bugs in the events database
granularity = 10000

### URL of the Debian bugs website
debbug_url = 'https://bugs.debian.org'

### Initialize the SOAP server for accessing the Debian bugs database
url = '%s/cgi-bin/soap.cgi' % debbug_url
namespace = 'Debbugs/SOAP'
server = SOAPpy.SOAPProxy (url, namespace)

### Get the newest bug number
newest_bug = server.newest_bugs (1) [0]

### Read the bugs database
bug_events_file = 'data/events/bugs'
input = file (bug_events_file).read ().decode ('utf-8').split ('\n')
current_bugs = []
bug_list = deb822.Deb822.iter_paragraphs (input, use_apt_pkg = False)

### Initialize the array for storing the information on the bugs
bug_paras = []

### Regular expression for getting the bug number from the source field
debbug_url_re = re.compile ('%s/(\d+)' % debbug_url)

### Iterate over the paragraphs, storing the bug numbers and building
### their string representation
for para in bug_list:
    m = debbug_url_re.search (para ['source'])
    if m:
        current_bugs.append (int (m.group (1)))
    bug_paras.append ('''Title: %s
Date: %s
Source: %s''' % (para ['title'], para ['date'], para ['source']))

### Get the bug recorded with highest number
last_recorded_bug = max (current_bugs)

### Regular expression for extracting the full name of the bug origniator
email_re = re.compile ('([^<]+)')

### Check whether there are new bugs to retrieve
if newest_bug >= last_recorded_bug + granularity:

    ## Yes : get them
    for i in range (1, 1 + (newest_bug - last_recorded_bug) / granularity):

        ## Get next bug
        bug = last_recorded_bug + i * granularity
        status = server.get_status (bug) [0]

        ## Get the full name of the submitter
        originator = status.value.originator
        m = email_re.search (originator)
        if m:
            originator = m.group (1).rstrip ()

        ## Get the date of submission
        date =  datetime.fromtimestamp (status.value.date).strftime ("%b %d %Y")

        ## Add paragraph to the database
        bug_paras.append ('\n'.join (['Title: Debian Bug #%d reported by %s' % (bug, originator),
                                      'Date: %s' % date,
                                      'Source: %s/%d' % (debbug_url, bug)]))
        print 'New bug found: #%d' % bug

    ## Write the new database
    output = open (bug_events_file, 'w')
    output.write ('\n\n'.join (bug_paras).encode ('utf-8') + '\n')
    output.close ()

else:
    ## No: nothing to do
    print "No new bugs."
